************************************************************
*****                                                  *****
***** DPC-Software Webcast, 26.06.2025                 *****
***** "Nützliche Stata-Befehle, die man kennen sollte" *****
***** Beispiel-Do-File                                 *****
***** by C. Brzinsky-Fay                               *****
*****                                                  *****
***** Verwendete Datensätze:                           *****
***** - data1.dta, uegdp1.dta, allstates.dta, epl1.dta *****            
*****                                                  *****
************************************************************
version 18
clear
set more off
capture log close
cd "c:\ado\webcast04"

*   ------------------------
**# 1) Neue Do-File Features
*   ------------------------
use "data1.dta", clear
/*
a)	Automatisches Vervollständigen
	Beim Tippen von Befehlen wird über ein Kontext-Fenster eine Auswahl für 
	Befehle und Variablennamen und Macros angeboten. Bei Befehlen, die interne
	Ergebnisse speichern, wird ebenfalls für die stored results eine angepasste
	Auswahlliste gezeigt.
	Die Auswahl erfolgt mit der Maus oder mit den Pfeiltasten.
*/
summarize yedu ybirth
tabulate ybirth
correlate hhinc rent
display r(rho)
generate kids=hhsize0to14>0
regress lsat i.sex hsat i.kids
display e(r2)

/*   
b)	Navigator-Leiste (rechts):
	Gesetzte Bookmarks können über die Navigator-Leiste schnell angesteuert
	werden.

c)	Highlighting von gleichen Ausdrücken und Klammern:
	Während des Schreibens in einem Do-Files werden gleich geschriebene 
	Ausdrücke hervorgehoben. Das erleichtert die Suche im Do-File nach z.B.
	Variablennamen oder Labels. Beispielsweise kann man so während des 
	Schreibens schneller kontrollieren, ob selbst definierte Variablennamen
	bei der weiteren Verwendung korrekt getippt werden.
	Auch Klammern werden entsprechend hervorgehoben, damit schneller überprüft
	werden kann, ob bei mehrfacher Schachtelung die "Klammerpaare" korrekt
	gesetzt werden.

d)	Leerzeichen- ("Whitespaces") und Tabulator-Markierungen:
	Bisher konnten Leerzeichen-Punkte und Tabulator-Pfeile nur an- bzw. aus-
	geschaltet werden. Mit Stata 19 erscheinen sie in markierten Textbereichen.

e)	Bereiche ein- bzw. ausfalten ("Code folding"):
	Einzelne Textbereiche im Do-File können ein- und ausgeklappt werden.
*/
forvalues num=1/9 {
	rename wor0`num' sorgen`num'
	tabulate sorgen`num' sex, col nofreq
}
/*
f)	Permanente und temporäre Bookmarks:
	Bookmarks waren bisher immer dauerhaft in Do-Files verankert. Jetzt können
	Anwender:innen auch temporäre Bookmarks setzen, die beim erneuten Öffnen
	eines Do-Files nicht mehr vorhanden sind.
*/

*   -----------------
**# 2) Stored Results
*   -----------------
use "data1.dta", clear
summarize ybirth
return list
*~~~~~ centering
use "data1.dta", clear
summarize rent
histogram rent, frequency
quietly summarize rent
generate rent_c=rent-r(mean)
histogram rent, ///
	frequency ///
	color(erose%50) ///
	addplot(histogram rent_c, frequency color(sand%50))
*~~~~~ standardising
summarize ybirth
generate ybirth_std=(ybirth-r(mean))/r(sd)
graph twoway scatter ybirth_std ybirth
*~~~~~ inversion
summarize lsat
generate lsat_inv=(r(max)+1)-lsat
histogram lsat, frequency discrete ///
	color(lavender%25) ///
	addplot(histogram lsat_inv, frequency discrete ///
		color(mint%25))
*~~~~~ confidence interval
summarize income
display r(mean)+1.96*sqrt(r(Var)/r(N))
display r(mean)-1.96*sqrt(r(Var)/r(N))
*~~~~~ using local macros
summarize income
local x=r(mean)
display `x'
summarize income if sex==1
local mean_men=r(mean)
summarize income if sex==2
display r(mean)-`mean_men'


*   ----------------------
**# 3) Regression & Macros
*   ----------------------
*~~~~~ a) preparation of regression models
use "data1.dta", clear
generate female=sex==2
generate east=state>=11 & state<=16 & state~=.
generate married=mar==1
summarize yedu
generate yedu_c=yedu-r(mean)
summarize ybirth
generate ybirth_c=ybirth-r(mean)
generate heval_c=heval-3
generate dsat_c=dsat-5
*~~~~~ b) estimation regression models & store results in macros
regress lsat female yedu_c ybirth_c
local modelfit1=e(r2)
regress lsat female yedu_c ybirth_c heval_c
local modelfit2=e(r2)
regress lsat female yedu_c ybirth_c heval_c east 
local modelfit3=e(r2)
regress lsat female yedu_c ybirth_c heval_c east married
local modelfit4=e(r2)
regress lsat female yedu_c ybirth_c heval_c east married dsat_c
local modelfit5=e(r2)
*~~~~~ c) create new dataset with r2 from the four regression models
clear
set obs 5
generate regmodel=_n
generate model_r2=.
forvalues model=1/5 {
	replace model_r2=`modelfit`model'' in `model'
}
label define lbregmodel 1 "M1" 2 "M2" 3 "M3" 4 "M4" 5 "M5"
label values regmodel lbregmodel
*~~~~~ d) create bar chart of these variables
graph bar model_r2, ///
	over(regmodel) ///
	blabel(bar, format(%7.3f)) ///
	bar(1, color(lavender)) ///
	ytitle("Determination Coefficients") ///
	ylabel(0.1(0.1)0.5) ///
	title("Model Statistics")


*   --------
**# 4) Loops
*-----------
*~~~~~ looping across numbers
forvalues num=1/10 {
	generate r`num'=runiform()
}
*~~~~~ looping across other units: variable names
use "data1.dta", clear
foreach X of varlist wor01-wor12 {
	tabulate `X' sex, nofreq column
}
*~~~~~ looping across other units: any strings
foreach word in We all live in the yellow submarine { 
	display "`word'" 
}
*~~~~~ looping with parallel lists
use "data1.dta", clear
local i=1
foreach var of varlist eqp* {
	generate equip`i' = `var' - 1 
	local i = `i' + 1 
}


*   --------------------
**# 5) by-Prefix & _n/_N
*   --------------------
*~~~~~ a simple grouping example
use "uegdp1.dta", clear
list, sepby(country)
by country (year), sort: generate counter=_n
list, sepby(country)
*~~~~~ 
use "data1.dta", clear
generate index=_n
list persnr intnr index in 1/20
display _N
by intnr, sort: generate intcount=_N
list index persnr intnr intcount in 1/30, sepby(intnr)

*   -------------------
**# 6) Clever functions
*   -------------------
use "data1.dta", clear
summarize income if inlist(mar,1,3,5)
summarize income if !inlist(emp,5)
summarize income if inrange(lsat,8,10)
summarize lsat if !inrange(ybirth,1984,1992)
summarize lsat if !inrange(ybirth,1984,1992)
generate wor_env=cond(wor05<wor06,wor05,wor06)
generate incnotemp=cond(emp==5,income,.)
generate hhincgrp=irecode(hhinc,10000,25000,50000,100000)
tabulate hhincgrp

*   ----------------
**# 7) egen-examples
*   ----------------
use "data1.dta", clear
egen worryind=anycount(wor*), values(1)
tabulate worryind
egen worrymiss=rowmiss(wor*)
tabulate worryind if worrymiss==0
egen inccat=cut(income), group(50)
graph hbar income, over(inccat)
egen egpsex=group(egp sex), label
tabulate egpsex
tabstat income, by(egpsex) labelwidth(40)

*   -----------------------
**# 8) Opacity/Transparency
*   -----------------------
use "data1.dta", clear
keep if income>0 & income<100000
histogram income, percent width(1000) by(sex, cols(1))
histogram income if sex==1, ///
	frequency ///
	color(ebblue%50) ///
	width(1000) ///
	addplot(histogram income if sex==2, ///
		frequency color(gold%50) width(1000)) ///
	legend(off)

*   -----------------------
**# 9) Weighted data points
*   -----------------------
use "allstates.dta", clear
graph twoway scatter ownhome propval100 [aweight=popden]
graph twoway scatter ownhome propval100 [aweight=popden], ///
	msize(small) mcolor(emerald%30)

*-----------------------
**# 10) Labels & Markers
*-----------------------
use "allstates.dta", clear
graph twoway ///
	(scatter divorced urban) ///
	(lfit divorced urban)
graph twoway ///
	(scatter divorced urban, mlabel(stateab)) ///
	(lfit divorced urban)
graph twoway ///
	(scatter divorced urban, mlabel(stateab) mlabposition(12)) ///
	(lfit divorced urban)
graph twoway ///
	(scatter divorced urban, mlabel(stateab) mlabposition(0)) ///
	(lfit divorced urban)
graph twoway ///
	(scatter divorced urban, mlabel(stateab) mlabposition(0) msymbol(i)) ///
	(lfit divorced urban)

*   ------------------------
**# 11) Distinct Data Series
*   ------------------------
use "uegdp1.dta", clear
graph twoway ///
	(lfit uer15_19 gdpgrowth) ///
	(scatter uer15_19 gdpgrowth, mcolor(black%20))
separate uer15_19, by(country) generate(yuer_)
labelbook lbcoun2
graph twoway ///
	(lfit uer15_19 gdpgrowth, lcolor(black)) ///
	(scatter uer15_19 gdpgrowth, mcolor(gs12%30)) ///
	(lfit yuer_10 gdpgrowth, lcolor(green)) ///
	(scatter yuer_10 gdpgrowth, ///
		mlabel(country2) ///
		mlabsize(vsmall) ///
		mlabposition(0) ///
		mlabcolor(green) ///
		msymbol(i)) ///
	(lfit yuer_9 gdpgrowth, lcolor(red)) ///
	(scatter yuer_9 gdpgrowth, ///
		mlabel(country2) ///
		mlabsize(vsmall) ///
		mlabposition(0) ///
		mlabcolor(red) ///
		msymbol(i)), ///
			legend(off)

*   ------------
**# 12) Fab-plot
*   ------------
ssc install fabplot
*~~~~~ example 1: spaghetti-problem
use "epl1.dta", clear
separate epl1, by(country) generate(epl_) shortlabel
graph twoway line epl_* year, ///
	legend(position(3) ///
		cols(1) ///
		size(vsmall))
*~~~~~ dissatisfying solution
graph twoway line epl1 year, ///
	by(country) ytitle("EPL index, version 1")
*~~~~~ better solution: front-and-back-plot (fab-plot)
fabplot line epl1 year, ///
	by(country) ///
	ytitle("EPL index, version 1")
*~~~~~ front-and-back plot (line) with different style front graph types
*~~~~~ i.e. option "front()"
fabplot line epl1 year, ///
	by(country) ///
	ytitle("EPL index, version 1") ///
	front(connect)
fabplot line epl1 year, ///
	by(country) ///
	ytitle("EPL index, version 1") ///
	front(scatter)
*~~~~~ front-and-back plots (line) with thick resp. coloured front line
*~~~~~ i.e. option "frontopts()"
fabplot line epl1 year, ///
	by(country) ///
	ytitle("EPL index, version 1") ///
	frontopts(lwidth(thick))
fabplot line epl1 year, ///
	by(country) ///
	ytitle("EPL index, version 1") ///
	frontopts(lwidth(medthick) lcolor(red))
*~~~~~ front-and-back plot (line) with selected front panels
*~~~~~ (but losing other panels for the back)
*~~~~~ using if-qualifier
fabplot line epl1 year ///
	if country==9 | country==19 | country==23 | country==29, ///
	by(country) ///
	ytitle("EPL index, version 1")
*~~~~~ front-and-back plot (line) with selected front panels
*~~~~~ (while keeping ALL back panels!)
*~~~~~ i.e. option(select)
generate wanted=inlist(country, 9, 19, 23, 29)
fabplot line epl1 year, ///
	by(country) ///
	ytitle("EPL index, version 1") ///
	select(wanted==1)
*~~~~~ example 2: the paella-problem
use "uegdp1.dta", clear
separate gdpgrowth, by(region) generate(gdp_) shortlabel
graph twoway scatter gdp_* uer15_19
*~~~~~ dissatisfying solution #1: using opacity
graph twoway scatter gdp_* uer15_19, ///
	mcolor(%25 %25 %25 %25)
*~~~~~ dissatisfying solution #2: by-option
graph twoway scatter gdpgrowth uer15_19, ///
	by(region)
*~~~~~ better solution: basic front-and-back plot (scatter)
fabplot scatter gdpgrowth uer15_19, ///
	by(region)
*~~~~~ front- and back-plot (scatter) without markers but labels only
fabplot scatter gdpgrowth uer15_19, ///
	by(country2) ///
	mcolor(gs12%50) ///
	frontopts( ///
		msymbol(none) ///
		mlabposition(0) ///
		mlabel(country2))
*~~~~~ front- and back-plot (scatter) without markers but labels only
*~~~~~ selected front panels (while keeping ALL back panels)
*~~~~~ REMARK: option needvars() not documented (Email of Nicolas J. Cox)
fabplot scatter gdpgrowth uer15_19, ///
	by(region) ///
	needvars(country2) ///
	mcolor(gs12%50) ///
	frontopts( ///
		msymbol(none) ///
		mlabposition(0) ///
		mlabel(country2) ///
		mlabsize(tiny))

*   -------------------------------------
**# 13) graph bar with CIs & unit symbols
*   -------------------------------------
use "data1.dta", clear
*~~~~~ bar chart with confidence intervals
graph hbar (meanci) lsat, over(state)
*~~~~~ bar charts with unit symbols
graph hbar income, over(state) blabel(bar, format(%9.2f) suffix(€))
graph bar, over(mar) blabel(bar, format(%5.1f) suffix(%))
